from soultrip.data import loadimage
from soultrip.constants import *
from soultrip.sacrifice import Sacrifice
import pygame
from pygame import Surface
import random



BG_HEIGHT = 720
SKY_HEIGHT = 4500

class Background(object):
    def __init__(self, speed, max_height):
        self.stars_bgs = [
            loadimage('sky-1.jpg').convert(),
            loadimage('sky-2.jpg').convert(),
            loadimage('sky-3.jpg').convert()
        ]

        self.stars_offset = 0
        self.twinkletimer = 0
        self.twinkle_time = 10

        self.inf_backgrounds = [
            loadimage('castle-wall-1.png').convert_alpha(),
            loadimage('castle-wall-2.png').convert_alpha(),
            loadimage('castle-wall-3.png').convert_alpha()
        ]
        self.images = []
        self.speed = speed
        self.height = 0
        self.ground_image = loadimage('groundroom.png').convert_alpha()
        self.roof_image = loadimage('castle-roof.png').convert_alpha()
        self.images.append(self.ground_image)
        self.images.append(random.choice(self.inf_backgrounds))
        self.offset = 0

        self.winning = False
        self.losing = False
        self.max_height = max_height

        self.sacrifice = Sacrifice()

        self.finished = False

        self.roof_added = False

    def reset(self):
        self.winning = False
        self.sacrifice.dead()
        self.sacrifice.win = False
        self.roof_added = False
        self.finished = False
        self.losing = False
        self.images = []
        self.images.append(self.ground_image)
        self.images.append(random.choice(self.inf_backgrounds))
        self.offset = 0
        self.height = 0


    def winupdate(self, dt):
        if not self.winning:
            self.winning = True
            passed = int(self.height / BG_HEIGHT)
            self.images = [self.ground_image] + [random.choice(self.inf_backgrounds) for x in range(0, passed)] + self.images
            self.offset += BG_HEIGHT * (passed + 1)

        if self.offset > 0:
            self.offset -= self.speed * dt
        if self.offset < 0:
            self.offset = 0

        if self.offset < 500:
            if self.sacrifice.status == 'dead':
                self.sacrifice.awake()
            self.sacrifice.offset = self.offset
            self.sacrifice.update(dt)
        if self.sacrifice.win:
            self.finished = True
        self.speed *= 1.05


    def loseupdate(self, dt):
        if not self.losing:
            self.losing = True

        if not self.roof_added:
            self.images.append(self.roof_image)
            self.roof_added = True

        
        self.offset += self.speed * dt

        if len(self.images) > 1:
            if self.offset > RESOLUTION_Y:
                self.offset = 0
                self.images = self.images[1:]


        else:
            if self.offset > RESOLUTION_Y + BG_HEIGHT:
                self.finished = True





    def update(self, dt):
        self.offset += self.speed * dt
        self.height += self.speed * dt

        

        if self.offset > BG_HEIGHT and not self.losing:
            self.images.append(random.choice(self.inf_backgrounds))
            self.images = self.images[1:]
            self.offset = 0

        if self.offset > BG_HEIGHT and self.losing:
            self.offset = 0

        if self.twinkletimer > self.twinkle_time:           
            if self.stars_offset < len(self.stars_bgs) - 1:
                self.stars_offset += 1
            else:
                self.stars_offset = 0
            self.twinkletimer = 0
        else:
            self.twinkletimer += dt
        if self.height < 500:
            self.sacrifice.offset = self.offset
            self.sacrifice.update(dt)

    def draw(self, surface):
        #yoffset = -SKY_HEIGHT + RESOLUTION_Y + self.height
        #surface.blit(self.stars_bgs[self.stars_offset], (0, 0))
        self.stars_bgs[self.stars_offset].set_alpha(255 * self.twinkletimer / self.twinkle_time)
        surface.blit(self.stars_bgs[self.stars_offset - 1], (0, 0))
        surface.blit(self.stars_bgs[self.stars_offset], (0, 0))
        for i,img in enumerate(self.images):
            if i == 0:
                surface.blit(img, (0, self.offset))
            else:
                surface.blit(img, (0, (-BG_HEIGHT * i) + self.offset))

        if self.height < 300 or (self.offset < 300 and self.winning):
            self.sacrifice.draw(surface)