from pygame import Surface
from soultrip.constants import *
from soultrip.inputmanager import InputManager
from soultrip.player import Player
from soultrip.stage import Stage
from soultrip.data import loadimage

class Game(object):
    def __init__(self):
        #self.start()
        self.exit = False

        self.beginning = True
        self.level_card = False
        self.title_card = loadimage('soultrip-titlecard.png').convert()
        self.credits_card = loadimage('soultrip-creditscard.png').convert()
        self.fade_in = True
        self.fade_out = False
        self.credits = False
        self.fade = 0.0
        
        self.waiting = True

        self.p = False
        self.s = False
        self.i = False

        self.switch_now = False

        self.show_card = 0

        self.cards = [
            loadimage('card-level1.png').convert_alpha(),
            loadimage('card-level2.png').convert_alpha(),
            loadimage('card-level3.png').convert_alpha(),
            loadimage('card-level4.png').convert_alpha(),
            loadimage('card-level5.png').convert_alpha(),
            loadimage('card-level6.png').convert_alpha(),
            loadimage('card-level7.png').convert_alpha()
        ]


        self.p = Player()
        self.i = InputManager(self.p)

    def start(self):
        self.s = Stage(self.p)

    def update(self, dt):
        if not self.waiting and self.s and self.s.speed < 1:
            self.s.speed = 5

        if self.switch_now:
            if self.beginning:
                self.level_card = True
                self.beginning = False
                self.start()
                self.fade_out = False
                self.fade_in = False
                self.fade = 0
            self.switch_now = False

        if self.s and self.s.finished:
            if self.s.finished and self.s.losing:
                self.credits = True
                self.beginning = False
            self.show_card += 1
            if self.show_card > len(self.cards) - 1:
                self.show_card = len(self.cards) - 1
            self.waiting = True
            self.level_card = True
            self.s.next_level()

        if self.s and self.s.losing and len(self.s.bg.images) < 2:
            self.fade = .1
            self.fade_out = True
            self.fade_in = False

        if self.p:
            self.p.waiting = self.waiting
        if self.s:
            self.s.waiting = self.waiting
            self.s.update(dt)

        if self.i:
            self.i.update(dt)
            if self.i.exit:
                self.exit = True

        if self.beginning:
            pass

        if self.fade_in:
            self.fade += .1 * dt

        if self.fade_out:
            self.fade -= .1 * dt

        if self.fade_in and self.fade > 1:
            self.fade_in = False
            self.fade = 5
            self.fade_out = True

        if self.fade_out and self.fade < 0:
            self.fade = 1
            self.fade_out = False
            self.fade_in = True
            self.switch_now = True

        if self.waiting and self.i.go:
            self.waiting = False
            self.level_card = False


    def draw(self, screen):
        if self.s:
            self.s.draw(screen)
        if self.beginning and not self.switch_now:
            #self.title_card.set_alpha(self.fade)
            screen.blit(self.title_card, (0,0))

        if self.credits:
            self.beginning = False
            self.s = False
            screen.blit(self.credits_card, (0,0))
            return


        alpha = 255 - (min(self.fade, 1)* 255)
        if alpha < 0:
            alpha = 0
        if alpha > 255:
            alpha = 255
        if alpha < 255:
            s = Surface((RESOLUTION_X, RESOLUTION_Y)).convert_alpha()
            s.fill((0,0,0, alpha))
            screen.blit(s, (0, 0))

        if self.level_card:
            screen.blit(self.cards[self.show_card], (0,0))