import pygame.sprite
from euclid import Vector2
from soultrip.data import loadimage

class GameSprite(pygame.sprite.Sprite):
    def __init__(self, image, scale, rect, animated):
        
        if scale is not 1:
            rect.width *= scale
            rect.height *= scale
            if animated != False:
                self.image = pygame.transform.smoothscale(loadimage(image), (rect.width * animated['frames'], rect.height))
            else:
                self.image = pygame.transform.smoothscale(loadimage(image), (rect.width, rect.height))
        else:
            self.image = loadimage(image)

        self.image.convert_alpha()

        self.rect = rect
        self.image_size = self.image.get_size()

        if animated != False:
            self.animated = True
            self.frames = animated['frames']
            self.fps = animated['fps']
            self.curr_frame = 1
            self.time_since_switch = 0
            self.display_image = pygame.Surface((self.rect.width * self.frames, self.rect.height)).convert_alpha()
        else:
            self.animated = False
            self.display_image = pygame.Surface((self.rect.width, self.rect.height)).convert_alpha()
        
        self.manual = False

    def update(self, dt):
        if self.animated and not self.manual:
            self.time_since_switch += dt
            if self.time_since_switch > self.fps:
                if self.curr_frame < self.frames:
                    self.curr_frame += 1
                else:
                    self.curr_frame = 1
                self.time_since_switch = 0

    def draw(self, screen):
   
        if self.animated:
            animoffset = pygame.Rect([self.rect.width * (self.curr_frame-1), 0, self.rect.width, self.rect.height])
            screen.blit(self.image, self.rect, animoffset)
        else:
            screen.blit(self.image, self.rect)
       