import pygame
import random
from pygame import Rect, Surface
from soultrip.constants import *
from soultrip.stageobject import StageObject
from euclid import Vector2
#import Numeric

RUNESIZE = 55
RUNES = [
	0x16A0,
	0x16A1,
	0x16A3,
	0x16A9,
	0x16C3,
	0x16C9,
	0x16D7,
	0x16D2,
	0x16D8,
	0x16E3,
	0x16EF,
	0x16E2,
	0x16EA
]
RUNES += range(0x16A5, 0x16A7)
RUNES += range(0x16AB, 0x16BC)

RUNECOLORS = [
	[255, 3, 3],
	[255, 109, 3],
	#[255, 191, 3],
	[230, 0, 18]
]

LETTER_OFFSET = 25
pygame.font.init()
class Rune(StageObject):

	font = pygame.font.Font('data/Junicode.ttf', RUNESIZE)
	
	def __init__(self, pos, speed, length = 1):
		StageObject.__init__(self)
		self.length = length
		self.pos = pos
		self.letters = []
		self.color = [255, 3, 3]
		self.speed = speed
		self.active = True

		while len(self.letters) < length:
			letter = unichr(random.choice(RUNES))
			if letter not in self.letters:
				self.letters.append(letter)

		#print self.letters
		surfs = [Rune.font.render(x, True, self.color) for x in self.letters]
		self.width = int(max(surfs, key = lambda s: s.get_width()).get_width())
		self.height = int(reduce(lambda s1, s2: s1 + s2.get_height(), surfs, 0))
		self.height -= LETTER_OFFSET * len(self.letters)
		self.extents = (self.width, self.height)
		self.surf_templ = Surface((self.width, self.height)).convert_alpha()
		self.surf = self.surf_templ.copy()
		
		#self.letters = [unichr(random.choice(RUNES)) for x in range(0, length)]
		

	def render2surf(self):
		surfs = [Rune.font.render(x, False, self.color) for x in self.letters]
		self.surf.fill([0, 0, 0, 0])
		y = 0
		for s in surfs:
			self.surf.blit(s, (0, y))
			y += s.get_height()
			y -= LETTER_OFFSET

	def update(self, dt):
		StageObject.update(self, dt)
		self.pos -= (self.speed * dt)
		offscreen = self.pos.x > RESOLUTION_X
		offscreen = offscreen or self.pos.y + self.height < 0
		offscreen = offscreen or self.pos.x + self.width < 0
		if offscreen:
			self.active = False

	def draw(self, layer):
		#StageObject.draw(self, layer)
		self.render2surf()
		for i in range(1,5):
			alpha = (1.0/i)
			
			self.color = random.choice(RUNECOLORS)
			self.render2surf()
			
			s = self.surf_templ.copy()
			s.fill((255,255,255,0))
			s.blit(self.surf, (0,0))

			pixels_alpha = pygame.surfarray.pixels_alpha(s)
			pixels_alpha[...] = (pixels_alpha * alpha)
			del pixels_alpha

			x = self.pos.x + (i * 1 * self.speed.x) - self.width/2
			y = self.pos.y + (i * 1 * self.speed.y) - self.height/2
			layer.blit(s, (x, y))