from euclid import Vector2
import random
from pygame.mixer import Sound
from soultrip.constants import *
from soultrip.rune import Rune
from soultrip.meter import Meter
from soultrip.background import Background
from soultrip.orb import Orb
from soultrip.data import load

MAX_ORB_TIME = 40

class Stage(object):
    def __init__(self, player):

        self.playing = True
        self.player = player
        self.cutscene = False
        self.runes = []
        self.runes_max = 8

        self.time_since_last_rune = -15

        self.height = 0
        self.speed = 0
        
        self.level = 0
        self.levels = [
            {
                "startorbs": 5,
                "orbtarget": 10,
                "difficulty": 0,
                "speed": 5
            },
            {
                "startorbs": 7,
                "orbtarget": 15,
                "difficulty": .1,
                "speed": 5
            },
            {
                "startorbs": 5,
                "orbtarget": 15,
                "difficulty": .1,
                "speed": 7
            },
            {
                "startorbs": 3,
                "orbtarget": 15,
                "difficulty": .2,
                "speed": 12
            },
            {
                "startorbs": 4,
                "orbtarget": 20,
                "difficulty": .3,
                "speed": 12
            },
            {
                "startorbs": 4,
                "orbtarget": 20,
                "difficulty": .3,
                "speed": 12
            },
            {
                "startorbs": 2,
                "orbtarget": 20,
                "difficulty": .5,
                "speed": 12
            },
        ]

        orbtarget = self.levels[self.level]['orbtarget']
        startorbs = self.levels[self.level]['startorbs']


        self.height_max = 720 * 4

        self.bg = Background(self.speed, self.height_max)

        self.meter = Meter(orbtarget, orbs = startorbs)

        self.give_orbs = True
        self.orbs = []
        self.difficulty = 0
        self.orb_timer = 0
        self.orb_time = 20
        self.winning = False
        self.losing = False

        self.finished = False

        self.good_sound = Sound(load('good-sound.ogg'))
        self.bad_sound = Sound(load('bad-sound.ogg'))

        self.body_pos = Vector2(600, 600)

        self.music_playing = False
        self.music = Sound(load('soultrip.ogg'))
        self.win_music = Sound(load('soultrip-success.ogg'))
        self.lose_music = Sound(load('soultrip-failure.ogg'))

    def start(self, level):
        if level > len(self.levels - 1):
            self.difficulty *= 2
        self.bg.reset()
        self.player.orbs = 0

        

    def next_level(self):
        self.level += 1
        if self.level > len(self.levels) - 1:
            self.levels[-1]['difficulty'] *= 2
            self.level = len(self.levels) - 1
        self.finished = False
        self.winning = False
        orbtarget = self.levels[self.level]['orbtarget']
        startorbs = self.levels[self.level]['startorbs']

        self.difficulty = self.levels[self.level]['difficulty']
        self.speed = self.levels[self.level]['speed']

        self.player.show()
        self.player.show_face = 'default'
        self.player.alpha_factor = .3
        self.player.base_alpha = .3
        self.player.velocity = Vector2(0, -11)
        self.bg.reset()
        self.music.stop()
        self.music_playing = False

        self.meter = Meter(orbtarget, orbs = startorbs)


    def update(self, dt):

        if self.winning:
            self.runes = []
            self.orbs = []
            self.player.show_face = 'win'
            if self.player.pos != self.body_pos:
                dist = self.player.pos - self.body_pos
                if dist.magnitude() < 3:
                    self.player.pos = self.body_pos
                if dist.magnitude > 3:
                    vec = dist.normalized()
                    self.player.pos -= vec * 5
                    self.player.speed = Vector2()
                    self.player.trail.appendleft(self.player.pos)
            else:
                self.player.hide()

            self.bg.winupdate(dt)

            if self.bg.finished:
                self.finished = True
            return

        if self.losing:

            self.runes = []
            self.orbs = []
            self.player.show_face = 'lose'
            self.bg.loseupdate(dt)

            if self.bg.finished:
                self.player.hide()
                self.finished = True

            return


        self.player.update(dt)
        self.runes_max = (self.difficulty * 10)  + 5
        self.rune_spawn_time = 15 - (self.difficulty * 10 )
        if len(self.runes) < self.runes_max and self.time_since_last_rune > self.rune_spawn_time and not self.waiting:
            if self.difficulty > 0 and random.random() < self.difficulty:
                #failed your stealth roll
                self.fire_rune_at_player()
            else:
                self.fire_rune_up()

            self.time_since_last_rune = 0
        else:
            self.time_since_last_rune += dt

        self.orb_time = MAX_ORB_TIME - (self.difficulty * 10 + int(self.speed))
        if self.orb_timer > self.orb_time and not self.waiting:
            self.add_orb()
            self.orb_timer = 0
        else:
            self.orb_timer += dt

        for o in self.orbs:
            o.speed = Vector2(0, self.speed)
            o.update(dt)

        for r in self.runes:
            r.update(dt)

        if not self.player.hurt:
            rrects = [r.rect for r in self.runes]
            hit = self.player.rect.collidelistall(rrects)
            if len(hit) > 0:
                for h in hit:
                    self.bad_sound.play()
                    self.runes[h].active = False
                    self.player.hurt = True
                    self.meter.orbs -= 1
                    self.speed += 1

            orects = [o.rect for o in self.orbs]
            get = self.player.rect.collidelistall(orects)
            if len(get) > 0:
                for g in get:
                    self.good_sound.play()
                    self.orbs[g].active = False
                    self.player.got_orb = True
                    self.meter.orbs += 1

        self.meter.update(dt)
        if self.meter.win:
            self.winning = True
            self.music.stop()
            self.music_playing = False
            self.win_music.play()

        if self.meter.lose:
            self.losing = True
            self.bg.losing = True
            self.music.stop()
            self.music_playing = False
            self.lose_music.play()

        self.bg.speed = self.speed
        if not self.waiting:
            if not self.music_playing and not self.winning and not self.losing:
                self.music.play(loops = -1)
                self.music_playing = True
            self.bg.update(dt)
    
    def add_orb(self):
        pos = self.get_new_rune_pos()
        pos.y = -10
        self.orbs.append(Orb(pos, Vector2(0, self.speed)))

    def get_new_rune_pos(self):
        return Vector2(random.randint(WALL_SIZE, RESOLUTION_X - WALL_SIZE), 730)

    def fire_rune_at_player(self):
        pos = self.get_new_rune_pos()
        vec = (self.player.pos - pos).normalized()
        vec *= -random.randint(5,50)
        self.runes.append(Rune(pos, vec, random.randint(1,2)))

    def fire_rune_up(self):
        pos = self.get_new_rune_pos()
        vec = Vector2(0, -1)
        vec *= -random.randint(5,50 * self.difficulty + 35)
        self.runes.append(Rune(pos, vec, random.randint(1,5)))

    def draw(self, surface):
        self.bg.draw(surface)
        self.meter.draw(surface)
        self.player.draw(surface)

        self.runes = filter(lambda x: x.active, self.runes)
        self.orbs = filter(lambda x: x.active, self.orbs)

        for o in self.orbs:
            o.draw(surface)
        for r in self.runes:
            r.draw(surface)